#include <msp430.h>
#include "i2c.h"

/**
 * Kerr D. Wong
 * http://www.kerrywong.com
 *
 * D3232 RTC Example Using MSP430G2452
 */

#define HIGH 1
#define LOW  0
#define OUTPUT 1
#define INPUT 0

#define DATETIME_REG_ADDR 0x00
#define CONTROL_REG_ADDR 0x0E
#define STATUS_REG_ADDR 0x0F
#define AGING_REG_ADDR 0x10
#define TEMPARATURE_REG_ADDR 0x11
#define SRAM_ADDR 0x14

#define _BV(bit) (1 << (bit))

#define DS3232_I2C_ADDRESS 0x68

//pins on PORT 1
// 4-7seg display pins
#define pinOE  0
#define pinLatch  1
#define pinClk  2
#define pinData 3

//pins on PORT 2
#define pinSecondSense  1

//depends on how the 7-seg display
//is implemented, you will likely
//need to change this
const unsigned char NUMS[] = {
		255-252, 255-96, 255-218, 255-242, 255-102,
		255-182, 255-190, 255-224, 255-254, 255-246,
		255 //blank
};

typedef struct {
  unsigned char second; //00H
  unsigned char minute; //01H
  unsigned char hour; //02H
  unsigned char dayOfWeek; //03H
  unsigned char dayOfMonth; //04H
  unsigned char month; //05H
  unsigned char year; //06H
} Date;

Date dt;

/**
 * Stop the Watch Dog Timer
 */
void inline stopWatchDogTimer() {
    WDTCTL = WDTPW + WDTHOLD;
}

/**
 * Set the pin mode
 * portNo: port (1 or 2)
 * pinNo: pin number in the port specified
 * pinMode: INPUT/OUTPUT
 */
void pinMode(unsigned char portNo, unsigned char pinNo, unsigned char pinMode) {
    if (portNo == 1) {
        if (pinMode)
            P1DIR |= (1 << pinNo);
        else
            P1DIR &= ~(1 << pinNo);
    } else if (portNo == 2) {
        if (pinMode)
            P2DIR |= (1 << pinNo);
        else
            P2DIR &= ~(1 << pinNo);
    }
}

/**
 * Write HIGH or LOW to a pin previously set as output
 * portNo: port (1 or 2)
 * pinNo: pin number in the port specified
 * value: HIGH/LOW
 */
void digitalWrite(unsigned char portNo, unsigned char pinNo, unsigned char value) {
    if (portNo == 1) {
        if (value)
            P1OUT |= 1 << pinNo;
        else
            P1OUT &= ~(1 << pinNo);
    } else if (portNo == 2) {
        if (value)
            P2OUT |= 1 << pinNo;
        else
            P2OUT &= ~(1 << pinNo);
    }
}

/**
 * Read logic level from pin
 * portNo: port (1 or 2)
 * pinNo: pin number in the port specified
 */
unsigned char digitalRead(unsigned char portNo, unsigned char pinNo) {
    int result;

    result = 0;

    if (portNo == 1) {
        result = (P1IN & (1 << pinNo)) >> pinNo;
    } else if (portNo == 2) {
        result = (P2IN & (1 << pinNo)) >> pinNo;
    }

    return result;
}

/**
 * Shift out using Port 1
 */
void shiftOut1(unsigned char dataPin, unsigned char clockPin, unsigned char value)
{
  int i;

  for (i = 0; i < 8; i++)  {
    digitalWrite(1, dataPin, (value & _BV(i)));
    digitalWrite(1, clockPin, HIGH);
    digitalWrite(1, clockPin, LOW);
  }
}

//Display the curent time
//secInd is fed from the SQW pin (pin 5) which is used to create the 
//flashing effect for the second indicator ":"
void displayTime(unsigned char hour, unsigned char min, unsigned char secInd)
{
        unsigned char d1 = 0, d2 = 0, d3 = 0, d4 = 0;

        d1 = hour/10;
        d2 = hour - d1 * 10 ;
        d3 = min/10;
        d4 = min - d3 * 10;

        if (d1 == 0) d1 = 10;

        digitalWrite(1, pinLatch, LOW);
        shiftOut1(pinData, pinClk, NUMS[d4]);
        if (secInd > 0) {
        	shiftOut1(pinData, pinClk, NUMS[d3] - 1);
            shiftOut1(pinData, pinClk, NUMS[d2] - 1);
        } else {
            shiftOut1(pinData, pinClk, NUMS[d3]);
            shiftOut1(pinData, pinClk, NUMS[d2]);
        }

        shiftOut1(pinData, pinClk, NUMS[d1]);
        digitalWrite(1, pinLatch, HIGH);
}

// Convert normal decimal numbers to binary coded decimal
unsigned char decToBcd(unsigned char val)
{
  return ( (val/10*16) + (val%10) );
}

// Convert binary coded decimal to normal decimal numbers
unsigned char bcdToDec(unsigned char val)
{
  return ( (val/16*10) + (val%16) );
}

// Gets the date and time from the ds1307 and prints result
void getDateDS3232()
{
	i2c_start();
	i2c_write8(DS3232_I2C_ADDRESS << 1);
	i2c_write8(DATETIME_REG_ADDR);
	i2c_stop();

	i2c_start();
	i2c_write8(DS3232_I2C_ADDRESS << 1 | 1);

	dt.second     = bcdToDec(i2c_read8(0x00));
	dt.minute     = bcdToDec(i2c_read8(0x00));
	dt.hour       = bcdToDec(i2c_read8(0x00)); // Need to change this if 12 hour am/pm
	dt.dayOfWeek  = bcdToDec(i2c_read8(0x00)); // 0= Sunday
	dt.dayOfMonth = bcdToDec(i2c_read8(0x00));
	dt.month      = bcdToDec(i2c_read8(0x00));
	dt.year       = bcdToDec(i2c_read8(0x00));

	i2c_stop();
}

void setDate3232()
{
	i2c_start();
	i2c_write8(DS3232_I2C_ADDRESS << 1);
	i2c_write8(DATETIME_REG_ADDR);

	i2c_write8(decToBcd(00)); //sec
	i2c_write8(decToBcd(43)); //min
	i2c_write8(decToBcd(20)); //hour
	i2c_write8(decToBcd(7)); //day
	i2c_write8(decToBcd(05)); //date
	i2c_write8(decToBcd(01)); //month
	i2c_write8(decToBcd(13)); //year

	i2c_stop();
}

void setControlRegisters(){
    i2c_start();
	i2c_write8(DS3232_I2C_ADDRESS << 1);

	i2c_write8(CONTROL_REG_ADDR); //Goto register 0Eh
	i2c_write8(0x00);

	i2c_stop();
}

void setupRTC3232(){
	i2c_init();
	//setDate3232(); // call this initially to set the date
	setControlRegisters();
}

void init()
{
	stopWatchDogTimer();

	pinMode(1, pinOE, OUTPUT);
	pinMode(1, pinLatch, OUTPUT);
	pinMode(1, pinClk, OUTPUT);
	pinMode(1, pinData, OUTPUT);
	pinMode(2, pinSecondSense, INPUT);

	digitalWrite(1, pinOE, LOW);
	digitalWrite(1, pinData, HIGH);
	digitalWrite(1, pinLatch, HIGH);
	digitalWrite(1, pinClk, HIGH);

	setupRTC3232();
}

void main(void) {
	int r = 0;

	init();

	for(;;) {
		getDateDS3232();

		r = digitalRead(2, pinSecondSense);
		displayTime(dt.hour, dt.minute, r);
	}
}
